local AddonName, AddonTable = ...

local Addon = AddonTable.Addon
local Module = Addon:NewModule("Warn")

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName, false)

local AC = LibStub("AceConsole-3.0")
local AT = LibStub("AceTimer-3.0")

local warnings = {
  180,
   30,
    5,
}

local timers = {}

local playerTarget


local function FormatTime(time)
  return format("%d:%02d", time / 60, time % 60)
end

local function Warn(time)
  AC:Print(format(L["WARNING"], playerTarget.name, FormatTime(time)))
  timers[time] = nil
end

local function BuffAdded(_, unit)
  if unit:GetOwner().GUID == UnitGUID("player") then
    playerTarget = unit

    local timeleft = unit:GetExpirationTime() - GetTime()
    for i, time in pairs(warnings) do
      if time < timeleft then
        timers[time] = AT:ScheduleTimer(Warn, timeleft - time, time)
      end
    end
  end
end

local function BuffRemoved(_, unit)
  if unit == playerTarget then
    playerTarget = nil

    for k, timer in pairs(timers) do
      AT:CancelTimer(timer)
      timers[k] = nil
    end
  end
end

local function BuffRefreshed(self, unit)
  BuffRemoved(self, unit)
  BuffAdded(self, unit)
end

function Module:OnEnable()
  Addon.RegisterCallback(self, "AURA_GAINED", BuffAdded)
  Addon.RegisterCallback(self, "AURA_REFRESHED", BuffRefreshed)
  Addon.RegisterCallback(self, "AURA_LOST", BuffRemoved)

  for unit in Addon:IterateTargets() do
    BuffAdded(nil, unit)
  end
end

function Module:OnDisable()
  Addon.UnregisterAllCallbacks(self)

  if playerTarget ~= nil then
    BuffRemoved(nil, playerTarget)
  end
end
